@@ -24,11 +24,11 @@
  |
  |---------------------------------------------------------------------------
  |
- | $Author: ying $
- | $Date: 2005/04/01 00:05:42 $
+ | $Author: gpachner $
+ | $Date: 2007-08-09 13:10:09 -0600 (Thu, 09 Aug 2007) $
  |
- | $RCSfile: zError.h,v $
- | $Revision: 1.11 $
+ | $RCSfile$
+ | $Revision: 2129 $
  |
  |---------------------------------------------------------------------------
  |	This module is used to:
@@ -74,6 +74,11 @@ extern "C" {
 #define zERR_BAD_ADDRESS					20018 /* A bad user space (ring 3) address */
 #define zERR_VALUE_CHANGED					20019 /* the value to be modified is changed by other process */
 
+#define	zERR_HMC_LESS_THAN_TWICE_NSS_CACHE	20030 /* Value must be twice as large as MinBufferCacheSize */
+#define	zERR_HMC_USE_MINBUFFERCACHESIZE	  	20031 /* Use MinBufferCacheSize command instead */
+#define	zERR_HMC_HIGH_MEMORY_LESS_THAN_TWICE_NSS_CACHE	20032 /* To use Linux Cache you must have at least
+																twice as much 'high memory' as NSS cache.	*/
+
 /* Message, user transaction, and task errors */
 
 #define zERR_BAD_KEY						20051 /* key couldn't be found */
@@ -388,12 +393,15 @@ extern "C" {
 #define zERR_AUTH_SPACE_NOT_IMPLEMENTED		20872 /* this function not implemented in this authorization space */
 #define zERR_CANT_MODIFY_OPEN_FILE_BEAST	20873 /* cant delete an open file without rights */
 
-
 /* MORE volume and pool errors */
 #define zERR_POOL_CONSUMER_PREVENTED		20880 /* NEB consumer failed an change pool state request */
 #define zERR_VOLUME_CONSUMER_PREVENTED		20881 /* NEB consumer failed an change volume state request */
 
-	/* NWCS Errors */
+	/* NWCS Errors.  These errors should only be returned by NCS software.
+	 * NSS should use zERR_NCS_xxx errors so that know the source of
+	 * a error.
+	 */
+#define zERR_NWCS_SOFTWARE_LEVELS_NOTSYNCED	20888 /* A member of the cluster is not at same software level as the others */
 #define zERR_NWCS_OBJECT_NOT_FOUND			20889 /* eDirectory object not found */
 #define zERR_NWCS_DUPLICATE_POOL_NAME		20890 /* The pool name already exists within the cluster. Indicates that
 												   * a local pool on one of the servers already is using the
@@ -455,6 +463,9 @@ extern "C" {
 #define zERR_CANT_HARD_LINK_TO_NON_FILE		21004 /* not allowed to create hard links to beasts not derived from file */
 #define zERR_LINK_DEST_FILE_ALREADY_EXISTS	21005 /* the new name for the link already exists */
 #define zERR_TOO_MANY_HARD_LINKS			21006 /* the new hard link may exceed the on-disk size limits for the file's metadata */
+#define zERR_LINK_CREATION_NOT_ENABLED		21007 /* Creation of hardlinks not enabled for this volume */
+#define zERR_MUST_UPGRADE_TO_LINK			21008 /* must upgrade to perform this operation on this hardlink */
+#define zERR_INVALID_INTERNAL_LINK_STRUCTURE 21009 /* internal hardlink structure consistency detetected */
 
 /* NLM Registration errors */
 #define zERR_MODULE_NAME_ALREADY_USED		21050 /* the given NSS MODULE name is already in use */
@@ -547,6 +558,11 @@ extern "C" {
 #define zERR_XML_IS_BAD							21702 /* bad XML found during parsing */
 #define zERR_XML_IS_INCOMPLETE					21703 /* XML found without terminating tag */
 #define zERR_UNABLE_TO_GET_ENOUGH_SIZE			21704 /* The actual size created or expanded is less than what users ask for */
+#define zERR_UNABLE_TO_GET_EVMS_SEGMENTS		21705 /* Not enough available evms partition space to satisfy request.  */
+
+/* NCS related errors that only NSS should return */
+#define zERR_NCS_CVB_NOT_REGISTERED				21710 /* Operation is not allowed when CVB is not registered */
+#define zERR_NCS_SYMBOL_IMPORT_ERROR			21711 /* Could not import a required NCS routine */
 
 /* Rebuild/Verify errors */
 #define zERR_RAV_VOLUME_FLUSH_FAILURE			21720
@@ -874,6 +890,14 @@ extern "C" {
 #define zERR_SUPERBLOCK_NOT_ENOUGH			24867 /* Not enough valid super block headers */
 #define zERR_SUPERBLOCK_CORRUPTED			24868 /* Valid super block header has bad information in it */
 
+/* ZFS misc. Errors */
+#define zERR_POOL_TOO_LARGE					24870 /* Pool is larger that 8 Terabytes */
+#define zERR_POOL_EXPAND_SAME				24871 /* Physical pool is already size requested by expand request */
+#define zERR_POOL_EXPAND_LESS				24872 /* New size is less than current Physical pool size */
+#define zERR_POOL_EXPAND_PAST_EOF			24873 /* Expand request to a size greater than the device */
+#define zERR_POOL_EXPAND_NOT_FOUND			24874 /* Physical pool was not found */
+#define zERR_POOL_TOO_LARGE_24875			24875 /* Used by mistake in Linux OES1 FCS, SP1 and SP2 */
+
 /* Recovery Errors */
 #define	zERR_RECOVERY_TOO_MANY_UNCOMMITS	24880 /* Recovery could not uncommit all transactions (that it needed
 												   * to) before it hit the last checkpoint. */
