@@ -24,11 +24,11 @@
  |
  |---------------------------------------------------------------------------
  |
- | $Author: taysom $
- | $Date: 2005/12/23 20:58:22 $
+ | $Author: cteerlink $
+ | $Date: 2007-08-08 10:08:23 -0600 (Wed, 08 Aug 2007) $
  |
- | $RCSfile: zParams.h,v $
- | $Revision: 1.13 $
+ | $RCSfile$
+ | $Revision: 2127 $
  |
  |---------------------------------------------------------------------------
  |	This module is used to:
@@ -45,6 +45,12 @@
 extern "C" {
 #endif
 
+#ifdef __LP64__
+#define zPACKED	__attribute__((packed))
+#else
+#define zPACKED
+#endif
+
 /*=========================================================================
  *=========================================================================
  *	Defines for bit masks and values
@@ -207,7 +213,7 @@ extern "C" {
 #define zCREATE_OPEN_IF_THERE		0x00000001
 #define zCREATE_TRUNCATE_IF_THERE	0x00000002
 #define zCREATE_DELETE_IF_THERE		0x00000004
-#define zCREATE_KEEP_VFS_CACHE		0x00000100
+#define zCREATE_KEEP_VFS_CACHE		0x00000100	/* !! FOR INTERNAL USE ONLY !! */
 
 #ifdef _NSS_INTERNAL_
 	#define zVALID_CREATE_FLAGS \
@@ -228,7 +234,7 @@ extern "C" {
  *---------------------------------------------------------------------------*/
 #define zDELETE_PURGE_IMMEDIATE		0x00000001
 #define zDELETE_FORCE_DELETE		0x00000002
-#define zDELETE_KEEP_VFS_CACHE		0x00000100
+#define zDELETE_KEEP_VFS_CACHE		0x00000100	/* !! FOR INTERNAL USE ONLY !! */
 
 /*-------------------------------------------------------------------------
  *	Definition of file attributes
@@ -255,6 +261,7 @@ extern "C" {
 #define zFA_COMPRESS_FILE_IMMEDIATELY 0x02000000
 #define zFA_DATA_STREAM_IS_COMPRESSED 0x04000000 /* per data stream directory entry */
 #define zFA_DO_NOT_COMPRESS_FILE	  0x08000000
+#define zFA_HARDLINK				0x10000000	 /* if set, this is a hardlink beast, real inode is pointed to by FILE */
 #define zFA_CANT_COMPRESS_DATA_STREAM 0x20000000 /* can't save any space by compressiong this data stream */
 #define zFA_ATTR_ARCHIVE	 		0x40000000
 #define zFA_VOLATILE				0x80000000	/* Data is volatile (no oplocks) */
@@ -268,7 +275,7 @@ extern "C" {
 			 zFS_REMOTE_DATA_ACCESS | zFA_REMOTE_DATA_INHIBIT | \
 			 zFA_COMPRESS_FILE_IMMEDIATELY | \
 			 zFA_DO_NOT_COMPRESS_FILE | zFA_ATTR_ARCHIVE | \
-			 zFA_NOT_VIRTUAL_FILE | zFA_VOLATILE)
+			 zFA_NOT_VIRTUAL_FILE | zFA_VOLATILE | zFA_HARDLINK)
 
 	#define zFA_VALID_DIRECTORY_ATTRIBUTES	\
 			(zFA_VALID_FILE_ATTRIBUTES | zFA_SUBDIRECTORY)
@@ -721,7 +728,7 @@ enum zGetInfoMask_t
  *	Definition of linkFlags in zLink
  *---------------------------------------------------------------------------*/
 #define zLF_HARD_LINK					0x00000001
-#define zLF_KEEP_VFS_CACHE				0x00000100
+#define zLF_KEEP_VFS_CACHE				0x00000100	/* !! FOR INTERNAL USE ONLY !! */
 
 #ifdef _NSS_INTERNAL_
 	#define zVALID_LINK_FLAGS (zLF_HARD_LINK | zLF_KEEP_VFS_CACHE)
@@ -759,7 +766,7 @@ enum zGetInfoMask_t
 										 * match.  If neither is set,
 										 * nothing will match.
 										 */
-#define zMATCH_ALL				(~0)	/* A define combining all of the
+#define zMATCH_ALL		(0xffffffff)	/* A define combining all of the
 										 * above flags that matches all
 										 * file objects in the given directory.
 										 * NOTE: if the match flag is set
@@ -887,7 +894,7 @@ enum zGetInfoMask_t
 #define zNSPACE_DATA_STREAM				6
 #define zNSPACE_EXTENDED_ATTRIBUTE		7
 
-#define zNSPACE_INVALID					(-1)
+#define zNSPACE_INVALID					(0xffffffff)
 
 #define zNSPACE_DOS_MASK				(1<<zNSPACE_DOS)
 #define zNSPACE_MAC_MASK				(1<<zNSPACE_MAC)
@@ -938,6 +945,7 @@ enum zGetInfoMask_t
 #define zMODE_UTF8				0x40000000
 #define zMODE_DELETED			0x20000000
 #define zMODE_LINK				0x10000000
+#define zMODE_VFS_OPEN			0x08000000	/* !! FOR INTERNAL USE ONLY !! */
 
 
 /*-------------------------------------------------------------------------
@@ -946,7 +954,7 @@ enum zGetInfoMask_t
 #define zRENAME_ALLOW_RENAMES_TO_MYSELF		0x00000001
 #define zRENAME_THIS_NAME_SPACE_ONLY		0x00000004
 #define zRENAME_DELETE_FILE_IF_THERE		0x00000008
-#define zRENAME_KEEP_VFS_CACHE				0x00000100
+#define zRENAME_KEEP_VFS_CACHE				0x00000100	/* !! FOR INTERNAL USE ONLY !! */
 
 #ifdef _NSS_INTERNAL_
 	#define zRENAME_COMPATABILITY			0x00000002 /* Matches a traditional NW value */
@@ -1000,6 +1008,11 @@ enum zGetInfoMask_t
  *									Replaces changing to direct I/O mode.
  *									Conflicts with regular open modes of
  *									the file.
+ * zRR_PREVENT_FS_HOOKS				Internal only use bit, if set, don't call
+ *									FS_Hooks during open/close on this file.
+ *									Used to prevent virus scanning of files
+ *									that have been opened for scan only during
+ *									backup operations.
  *
  * zRR_TRANSACTION_ACTIVE			Have the file use the default transaction.
  *
@@ -1017,7 +1030,9 @@ enum zGetInfoMask_t
  *									information without causing a change
  *									in the files meta-data that would
  *									eventually have to be written.
+ *
  */
+
 #define zRR_READ_ACCESS						0x00000001
 #define zRR_WRITE_ACCESS					0x00000002
 #define zRR_DENY_READ						0x00000004
@@ -1036,6 +1051,7 @@ enum zGetInfoMask_t
 #define zRR_TRANSACTION_ACTIVE				0x00100000
 #define zRR_PSA_CACHE						0x00200000
 
+#define zRR_PREVENT_FS_HOOKS				0x01000000
 #define zRR_MASK_READ_IN_DROP_BOXES			0x02000000
 #define zRR_READ_ACCESS_TO_SNAPSHOT			0x04000000
 #define zRR_DENY_RW_OPENER_CAN_REOPEN		0x08000000
@@ -1162,6 +1178,7 @@ enum {
 #define zROOTDIR_ZID			((Zid_t)zFIRST_ALLOCATABLE_ZID-1)	/* ID of rootdir*/
 #define	zDOT_COOKIE		(zROOTDIR_ZID-1)	/* NFS cookie for "." */
 #define zDOTDOT_COOKIE	(zROOTDIR_ZID-2)	/* NFS cookie for ".." */
+#define zNOMORE_COOKIES	(zROOTDIR_ZID-3)	/* NFS no more files */
 
 /* ID of first ZID definable by users, all well known zids will be less than this */
 #define zFIRST_ALLOCATABLE_ZID	((Zid_t)zDEFAULT_ZID_ALLOCATION_RANGE)
@@ -1240,7 +1257,7 @@ typedef struct zVolumeInfo_s
 		QUAD	enableModMask;	/* zMOD_VOL_ATTRIBUTES
 								 * Mask of features that should be changed. */
 		QUAD	supported;		/* Features that can be supported by volume */
-	} features;
+	} zPACKED features;
 	QUAD		maximumFileSize;/* Maximum file size supported by volume. */	
 	QUAD		totalSpaceQuota;/* zMOD_VOL_QUOTA
 								 * Maximum size of this volume
@@ -1285,7 +1302,7 @@ typedef struct zVolumeInfo_s
 								 * below the lowWaterMark. */
 		LONG	lowWaterMark;		/* zMOD_VOL_LOW_WATER_MARK */
 		LONG	highWaterMark;		/* zMOD_VOL_HIGH_WATER_MARK */
-	} salvage;
+	} zPACKED salvage;
     struct {                            /* Compression-related Statistics */
         QUAD    numCompressedFiles;	/* Number of files that have been
 									 * compressed. */
@@ -1295,8 +1312,8 @@ typedef struct zVolumeInfo_s
         QUAD    numPreCompressedBytes;	/* How much space we would have used
 										 * without compression. */
         QUAD    numCompressedBytes;	/* How much space used with compressions. */
-    } comp;
-} zVolumeInfo_s;
+    } zPACKED comp;
+} zPACKED zVolumeInfo_s;
 
 typedef struct zPoolInfo_s
 {
@@ -1316,26 +1333,27 @@ typedef struct zPoolInfo_s
 								 * changed. */
 		QUAD	supported;		/* Features that can be supported by
 								 * this pool. */
-	} features;
+	} zPACKED features;
 	QUAD		totalSpace;		/* Total number of bytes available on
 								 * pool. */
 	QUAD		numUsedBytes;		/* Number of bytes used in pool. */
 	QUAD		purgeableBytes;		/* Number of bytes in deleted but not
 									 * purged files. */
 	QUAD		nonPurgeableBytes;	/* Deleted files but can't be purged yet. */
-} zPoolInfo_s;
+} zPACKED zPoolInfo_s;
 
 typedef struct zTrusteeInfo_s
 {
 	GUID_t		trustee;
 	LONG		rights;
-} zTrusteeInfo_s;
+} zPACKED zTrusteeInfo_s;
 
 enum
 {
 	zINFO_VERSION_A = 1,
 	zINFO_VERSION_B,
-	zINFO_VERSION_C
+	zINFO_VERSION_C,
+	zINFO_VERSION_D
 };
 
 
@@ -1351,7 +1369,7 @@ typedef struct Standard_s {
 	LONG		fileAttributes;			/* zMOD_FILE_ATTRIBUTES */
 	LONG		fileAttributesModMask;	/* zMOD_FILE_ATTRIBUTES */
 	LONG		padding;
-} Standard_s;
+} zPACKED Standard_s;
 
 	/*
 	 * Fields are arranged to make alignment easier (QUAD, LONG, WORD, BYTE)
@@ -1376,7 +1394,7 @@ typedef struct zInfo_s
 								 */
     	QUAD	 dataBytes;		/* Storage used by user data	*/
     	QUAD	 metaDataBytes;	/* Storage used by Meta-data	*/
-	} storageUsed;
+	} zPACKED storageUsed;
 
 	LONG	primaryNameSpaceID;		/* zGET_PRIMARY_NAMESPACE */
 									/* zMOD_PRIMARY_NAMESPACE */
@@ -1387,7 +1405,7 @@ typedef struct zInfo_s
 		LONG 	fileNameArray;		/* Index of the array of indicies in
 									 * the variableData area
 									 */
-	} names;
+	} zPACKED names;
 
 	struct {						/* zGET_TIMES */
 		QUAD	created;			/* zMOD_CREATED_TIME */
@@ -1395,51 +1413,51 @@ typedef struct zInfo_s
 		QUAD	modified;			/* zMOD_MODIFIED_TIME */
 		QUAD	accessed;			/* zMOD_ACCESSED_TIME */
 		QUAD	metaDataModified;	/* zMOD_METADATA_MODIFIED_TIME */
-	} time;
+	} zPACKED time;
 
 	struct {						/* zGET_IDS */
 	 	UserID_t owner;				/* zMOD_OWNER_ID */
     	UserID_t archiver;			/* zMOD_ARCHIVER_ID */
     	UserID_t modifier;			/* zMOD_MODIFIER_ID */
     	UserID_t metaDataModifier;	/* zMOD_METADATA_MODIFIER_ID */
-	} id;
+	} zPACKED id;
 
 	struct {						/* zGET_BLOCK_SIZE */
 		LONG	 size;
 		LONG	 sizeShift;
-	} blockSize;
+	} zPACKED blockSize;
 
 	struct {						/* zGET_COUNTS */
 		LONG	 open;
 		LONG	 hardLink;
-	} count;
+	} zPACKED count;
 
 	struct {						/* zGET_DATA_STREAM_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} dataStream;
+	} zPACKED dataStream;
 
 	struct {						/* zGET_EXTENDED_ATTRIBUTE_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} extAttr;
+	} zPACKED extAttr;
 
 	struct {						/* zGET_DELETED_INFO */
 									/* zMOD_DELETED_INFO */
 		QUAD	 time;
 		UserID_t id;
-	} deleted;
+	} zPACKED deleted;
 
 	struct {						/* zGET_MAC_METADATA */
 		zMacInfo_s info;			/* zMOD_MAC_METADATA */
-	} macNS;
+	} zPACKED macNS;
 
 	struct {						/* zGET_UNIX_METADATA */
 		zUnixInfo_s info;			/* zMOD_UNIX_METADATA */
 		LONG		offsetToData;
-	} unixNS;
+	} zPACKED unixNS;
 
 	zVolumeInfo_s	vol;			/* zGET_VOLUME_INFO */
 
@@ -1451,7 +1469,7 @@ typedef struct zInfo_s
 	/* The variable portion will contain all variable size optional data */
 	BYTE		variableData[zGET_INFO_VARIABLE_DATA_SIZE];
 
-} zInfo_s;
+} zPACKED zInfo_s;
 
 	/*
 	 * Fields are arranged to make alignment easier (QUAD, LONG, WORD, BYTE)
@@ -1478,7 +1496,7 @@ typedef struct zInfoB_s
 								 */
     	QUAD	 dataBytes;		/* Storage used by user data	*/
     	QUAD	 metaDataBytes;	/* Storage used by Meta-data	*/
-	} storageUsed;
+	} zPACKED storageUsed;
 
 	LONG	primaryNameSpaceID;		/* zGET_PRIMARY_NAMESPACE */
 									/* zMOD_PRIMARY_NAMESPACE */
@@ -1489,7 +1507,7 @@ typedef struct zInfoB_s
 		LONG 	fileNameArray;		/* Index of the array of indicies in
 									 * the variableData area
 									 */
-	} names;
+	} zPACKED names;
 
 	struct {						/* zGET_TIMES */
 		QUAD	created;			/* zMOD_CREATED_TIME */
@@ -1497,51 +1515,51 @@ typedef struct zInfoB_s
 		QUAD	modified;			/* zMOD_MODIFIED_TIME */
 		QUAD	accessed;			/* zMOD_ACCESSED_TIME */
 		QUAD	metaDataModified;	/* zMOD_METADATA_MODIFIED_TIME */
-	} time;
+	} zPACKED time;
 
 	struct {						/* zGET_IDS */
 	 	UserID_t owner;				/* zMOD_OWNER_ID */
     	UserID_t archiver;			/* zMOD_ARCHIVER_ID */
     	UserID_t modifier;			/* zMOD_MODIFIER_ID */
     	UserID_t metaDataModifier;	/* zMOD_METADATA_MODIFIER_ID */
-	} id;
+	} zPACKED id;
 
 	struct {						/* zGET_BLOCK_SIZE */
 		LONG	 size;
 		LONG	 sizeShift;
-	} blockSize;
+	} zPACKED blockSize;
 
 	struct {						/* zGET_COUNTS */
 		LONG	 open;
 		LONG	 hardLink;
-	} count;
+	} zPACKED count;
 
 	struct {						/* zGET_DATA_STREAM_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} dataStream;
+	} zPACKED dataStream;
 
 	struct {						/* zGET_EXTENDED_ATTRIBUTE_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} extAttr;
+	} zPACKED extAttr;
 
 	struct {						/* zGET_DELETED_INFO */
 									/* zMOD_DELETED_INFO */
 		QUAD	 time;
 		UserID_t id;
-	} deleted;
+	} zPACKED deleted;
 
 	struct {						/* zGET_MAC_METADATA */
 		zMacInfo_s info;			/* zMOD_MAC_METADATA */
-	} macNS;
+	} zPACKED macNS;
 
 	struct {						/* zGET_UNIX_METADATA */
 		zUnixInfo_s info;			/* zMOD_UNIX_METADATA */
 		LONG		offsetToData;
-	} unixNS;
+	} zPACKED unixNS;
 
 	zVolumeInfo_s	vol;			/* zGET_VOLUME_INFO */
 
@@ -1553,14 +1571,14 @@ typedef struct zInfoB_s
 	struct {						/* zGET_DIR_QUOTA */
 		SQUAD	quota;				/* zMOD_DIR_QUOTA */
 		SQUAD	usedAmount;
-	} dirQuota;
+	} zPACKED dirQuota;
 	
 									
 
 	/* The variable portion will contain all variable size optional data */
 	BYTE		variableData[zGET_INFO_VARIABLE_DATA_SIZE];
 
-} zInfoB_s;
+} zPACKED zInfoB_s;
 
 	/*
 	 * Fields are arranged to make alignment easier (QUAD, LONG, WORD, BYTE)
@@ -1585,7 +1603,7 @@ typedef struct zInfoC_s
 								 */
     	QUAD	 dataBytes;		/* Storage used by user data	*/
     	QUAD	 metaDataBytes;	/* Storage used by Meta-data	*/
-	} storageUsed;
+	} zPACKED storageUsed;
 
 	LONG	primaryNameSpaceID;		/* zGET_PRIMARY_NAMESPACE */
 									/* zMOD_PRIMARY_NAMESPACE */
@@ -1596,7 +1614,7 @@ typedef struct zInfoC_s
 		LONG 	fileNameArray;		/* Index of the array of indicies in
 									 * the variableData area
 									 */
-	} names;
+	} zPACKED names;
 
 	struct {						/* zGET_TIMES */
 		QUAD	created;			/* zMOD_CREATED_TIME */
@@ -1604,51 +1622,51 @@ typedef struct zInfoC_s
 		QUAD	modified;			/* zMOD_MODIFIED_TIME */
 		QUAD	accessed;			/* zMOD_ACCESSED_TIME */
 		QUAD	metaDataModified;	/* zMOD_METADATA_MODIFIED_TIME */
-	} time;
+	} zPACKED time;
 
 	struct {						/* zGET_IDS */
 	 	UserID_t owner;				/* zMOD_OWNER_ID */
     	UserID_t archiver;			/* zMOD_ARCHIVER_ID */
     	UserID_t modifier;			/* zMOD_MODIFIER_ID */
     	UserID_t metaDataModifier;	/* zMOD_METADATA_MODIFIER_ID */
-	} id;
+	} zPACKED id;
 
 	struct {						/* zGET_BLOCK_SIZE */
 		LONG	 size;
 		LONG	 sizeShift;
-	} blockSize;
+	} zPACKED blockSize;
 
 	struct {						/* zGET_COUNTS */
 		LONG	 open;
 		LONG	 hardLink;
-	} count;
+	} zPACKED count;
 
 	struct {						/* zGET_DATA_STREAM_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} dataStream;
+	} zPACKED dataStream;
 
 	struct {						/* zGET_EXTENDED_ATTRIBUTE_INFO */
 		LONG	 count;
 		LONG	 totalNameSize;
 		QUAD	 totalDataSize;
-	} extAttr;
+	} zPACKED extAttr;
 
 	struct {						/* zGET_DELETED_INFO */
 									/* zMOD_DELETED_INFO */
 		QUAD	 time;
 		UserID_t id;
-	} deleted;
+	} zPACKED deleted;
 
 	struct {						/* zGET_MAC_METADATA */
 		zMacInfo_s info;			/* zMOD_MAC_METADATA */
-	} macNS;
+	} zPACKED macNS;
 
 	struct {						/* zGET_UNIX_METADATA */
 		zUnixInfo_s info;			/* zMOD_UNIX_METADATA */
 		LONG		offsetToData;
-	} unixNS;
+	} zPACKED unixNS;
 
 	zVolumeInfo_s	vol;			/* zGET_VOLUME_INFO */
 
@@ -1660,7 +1678,7 @@ typedef struct zInfoC_s
 	struct {						/* zGET_DIR_QUOTA */
 		SQUAD	quota;				/* zMOD_DIR_QUOTA */
 		SQUAD	usedAmount;
-	} dirQuota;
+	} zPACKED dirQuota;
 	
 	LONG inheritedRightsMask;		/* zGET_INH_RIGHTS_MASK */
 									/* zMOD_INH_RIGHTS_MASK */
@@ -1671,12 +1689,134 @@ typedef struct zInfoC_s
 		LONG 	trusteeArray;		/* Index of the array of zTrusteeInfo_s
 									 * structures in the variableData area
 									 */
-	} trustees;
+	} zPACKED trustees;
 
 	/* The variable portion will contain all variable size optional data */
 	BYTE		variableData[zGET_INFO_VARIABLE_DATA_SIZE];
 
-} zInfoC_s;
+} zPACKED zInfoC_s;
+
+	/*
+	 * Fields are arranged to make alignment easier (QUAD, LONG, WORD, BYTE)
+	 */
+typedef struct zInfoD_s
+{
+	LONG	infoVersion;	/* Verion of Info structure			*/
+	SLONG	totalBytes;		/* Total bytes in fixed and variable
+							 * data sections.  We use both sections
+							 * so we can grow the fixed area without
+							 * breaking older code.
+							 */
+	SLONG	nextByte;		/* Next byte to use in variable area */
+	LONG	padding;
+	QUAD	retMask;		/* Fields set in Info structure		*/
+	Standard_s	std;				/* zGET_STD_INFO			*/
+
+	struct {						/* zGET_STORAGE_USED */
+		QUAD	 physicalEOF;	/* The file may have more blocks allocated
+								 * than it needs, this marks of the end of
+								 * extra data blocks
+								 */
+    	QUAD	 dataBytes;		/* Storage used by user data	*/
+    	QUAD	 metaDataBytes;	/* Storage used by Meta-data	*/
+	} zPACKED storageUsed;
+
+	LONG	primaryNameSpaceID;		/* zGET_PRIMARY_NAMESPACE */
+									/* zMOD_PRIMARY_NAMESPACE */
+	LONG 	nameStart;				/* zGET_NAME - index into variableData */
+
+	struct {						/* zGET_ALL_NAMES */
+		LONG 	numEntries;
+		LONG 	fileNameArray;		/* Index of the array of indicies in
+									 * the variableData area
+									 */
+	} zPACKED names;
+
+	struct {						/* zGET_TIMES */
+		QUAD	created;			/* zMOD_CREATED_TIME */
+		QUAD	archived;			/* zMOD_ARCHIVED_TIME */
+		QUAD	modified;			/* zMOD_MODIFIED_TIME */
+		QUAD	accessed;			/* zMOD_ACCESSED_TIME */
+		QUAD	metaDataModified;	/* zMOD_METADATA_MODIFIED_TIME */
+	} zPACKED time;
+
+	struct {						/* zGET_IDS */
+	 	UserID_t owner;				/* zMOD_OWNER_ID */
+    	UserID_t archiver;			/* zMOD_ARCHIVER_ID */
+    	UserID_t modifier;			/* zMOD_MODIFIER_ID */
+    	UserID_t metaDataModifier;	/* zMOD_METADATA_MODIFIER_ID */
+	} zPACKED id;
+
+	struct {						/* zGET_BLOCK_SIZE */
+		LONG	 size;
+		LONG	 sizeShift;
+	} zPACKED blockSize;
+
+	struct {						/* zGET_COUNTS */
+		LONG	 open;
+		LONG	 hardLink;
+	} zPACKED count;
+
+	struct {						/* zGET_DATA_STREAM_INFO */
+		LONG	 count;
+		LONG	 totalNameSize;
+		QUAD	 totalDataSize;
+	} zPACKED dataStream;
+
+	struct {						/* zGET_EXTENDED_ATTRIBUTE_INFO */
+		LONG	 count;
+		LONG	 totalNameSize;
+		QUAD	 totalDataSize;
+	} zPACKED extAttr;
+
+	struct {						/* zGET_DELETED_INFO */
+									/* zMOD_DELETED_INFO */
+		QUAD	 time;
+		UserID_t id;
+	} zPACKED deleted;
+
+	struct {						/* zGET_MAC_METADATA */
+		zMacInfo_s info;			/* zMOD_MAC_METADATA */
+	} zPACKED macNS;
+
+	struct {						/* zGET_UNIX_METADATA */
+		zUnixInfo_s info;			/* zMOD_UNIX_METADATA */
+		LONG		offsetToData;
+	} zPACKED unixNS;
+
+	zVolumeInfo_s	vol;			/* zGET_VOLUME_INFO */
+
+	zPoolInfo_s		pool;			/*	zGET_POOL_METADATA	*/
+
+	LONG	extAttrUserFlags;		/* zGET_EXTATTR_FLAGS */
+									/* zMOD_EXTATTR_FLAGS */
+
+	struct {						/* zGET_DIR_QUOTA */
+		SQUAD	quota;				/* zMOD_DIR_QUOTA */
+		SQUAD	usedAmount;
+	} zPACKED dirQuota;
+	
+	LONG inheritedRightsMask;		/* zGET_INH_RIGHTS_MASK */
+									/* zMOD_INH_RIGHTS_MASK */
+
+	struct {						/* zGET_ALL_TRUSTEES */
+									/* zMOD_ALL_TRUSTEES */
+		LONG 	numEntries;
+		LONG 	trusteeArray;		/* Index of the array of zTrusteeInfo_s
+									 * structures in the variableData area
+									 */
+	} zPACKED trustees;
+
+	Zid_t		hardlinkPrimaryZid;	/* zGET_STD_INFO
+									 * ZID of hardlink primary beast. Will be
+									 * zINVALID_ZID if this is not a hardlink.
+									 */
+
+	/* The variable portion will contain all variable size optional data */
+	BYTE		variableData[zGET_INFO_VARIABLE_DATA_SIZE];
+
+} zPACKED zInfoD_s;
+
 
 #define zINFO_PTR(_info, _i)	(&(((BYTE *)(_info))[(_i)]))
 
@@ -1696,7 +1836,7 @@ typedef struct ZnewExtAttrInfo_s
 {
 	NINT		extAttrUserFlags;		/* zGET_EXTATTR_FLAGS */
 										/* zMOD_EXTATTR_FLAGS */
-} ZnewExtAttrInfo_s;
+} zPACKED ZnewExtAttrInfo_s;
 
 /*---------------------------------------------------------------------------
  *	Definition of typeInfo returned for zFTYPE_SYM_LINK
@@ -1709,7 +1849,7 @@ typedef struct ZnewGetSymLinkInfo_s
 {
 	LONG	symLinkStart;	/* Start of path name in variableData */
 	BYTE	variableData[zGET_SYMLINK_VARIABLE_DATA_SIZE];
-} ZnewGetSymLinkInfo_s;
+} zPACKED ZnewGetSymLinkInfo_s;
 
 
 /*=========================================================================
@@ -1741,7 +1881,7 @@ typedef struct zGetName_s
 	/* The variable portion will contain all variable size optional data */
 	BYTE		variableData[zGET_NAME_VARIABLE_DATA_SIZE];
 
-} zGetName_s;
+} zPACKED zGetName_s;
 
 
 /*=========================================================================
@@ -1809,7 +1949,7 @@ typedef struct zGetSymLinkInfo_s
 {
 	unicode_t	*symLinkPath;
 	BYTE	variableData[zGET_SYMLINK_VARIABLE_DATA_SIZE];
-} zGetSymLinkInfo_s;
+} zPACKED zGetSymLinkInfo_s;
 
 
 /*=========================================================================
@@ -1827,7 +1967,7 @@ typedef struct zMatchAttr_s
 
 	NINT	matchTypeAttrSet;
 	NINT	matchTypeAttrNotSet;
-} zMatchAttr_s;
+} zPACKED zMatchAttr_s;
 
 
 
@@ -1853,7 +1993,7 @@ typedef struct zUserRest_s
 	UserID_t	userID;
 	SQUAD		usedAmount;
 	SQUAD		restrictionAmount;
-} zUserRest_s;
+} zPACKED zUserRest_s;
 
 
 /*---------------------------------------------------------------------------
@@ -1864,7 +2004,7 @@ typedef struct zTrustee_s
 	UserID_t	trustee;
 	LONG		rights;
 	LONG		padding;
-} zTrustee_s;
+} zPACKED zTrustee_s;
 
 /************************************************************************
  *  States of the virtual file
@@ -1873,6 +2013,7 @@ typedef struct zTrustee_s
 #define FHV_ACTUAL_DATA			0x01	/* set if we are working with the real data */
 #define FHV_RESULT_DATA			0x02	/* set when data in result buffer is good */
 #define FHV_FORCE_RESULT_READ	0x04	/* read from result buffer only */
+#define FHV_TEMPLATE_DEFINED	0x08	/* a template has been defined during this session */
 
 /*-------------------------------------------------------------------------
  *	Structure maintained in the file handle for a virtual file
